;*******************************************************************************************
;*
;*   raylib [core] example - 3d camera first person
;*
;*   This example has been created using raylib 1.3 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2015 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

#MAX_COLUMNS = 20

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [core] example - 3d camera first person" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

; Define the camera to look into our 3d world
Define camera.Camera3D
InitVector3( @camera\position, 4.0, 2.0, 4.0 )      ; Camera position
InitVector3( @camera\target  , 0.0, 1.8, 0.0 )      ; Camera looking at point
InitVector3( @camera\up      , 0.0, 1.0, 0.0 )      ; Camera up vector (rotation towards target)
camera\fovy = 60.0                                  ; Camera field-of-view Y
camera\type = #CAMERATYPE_PERSPECTIVE               ; Camera mode type

; Generates some random columns
Dim heights.f(#MAX_COLUMNS)
Dim positions.Vector3(#MAX_COLUMNS)
Dim colors.i(#MAX_COLUMNS)

Define i
For i = 0 To #MAX_COLUMNS - 1
    heights(i) = GetRandomValue(1, 12)
    InitVector3( @positions(i), GetRandomValue(-15, 15), heights(i)/2, GetRandomValue(-15, 15) )
    colors(i) = RGBA( GetRandomValue(20, 255), GetRandomValue(10, 55), 30, 255 )
Next

SetCameraMode(@camera, #CAMERAMODE_FIRST_PERSON)    ; Set a first person camera mode

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    UpdateCamera( @camera )             ; Update camera
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)
        
        BeginMode3D(@camera)
        
            Define vec.Vector3, vec2d.Vector2
            InitVector3(@vec, 0.0, 0.0, 0.0)
            InitVector2(@vec2d, 32.0, 32.0)
            DrawPlane(@vec, @vec2d, #COLOR_LIGHTGRAY)       ; Draw ground
            InitVector3(@vec, -16.0, 2.5, 0.0)
            DrawCube(@vec, 1.0, 5.0, 32.0, #COLOR_BLUE)     ; Draw a blue wall
            InitVector3(@vec, 16.0, 2.5, 0.0)
            DrawCube(@vec, 1.0, 5.0, 32.0, #COLOR_LIME)     ; Draw a green wall
            InitVector3(@vec, 0.0, 2.5, 16.0)
            DrawCube(@vec, 32.0, 5.0, 1.0, #COLOR_GOLD)     ; Draw a yellow wall
    
            ; Draw some cubes around
            For i = 0 To #MAX_COLUMNS - 1
                DrawCube     (@positions(i), 2.0, heights(i), 2.0, colors(i))
                DrawCubeWires(@positions(i), 2.0, heights(i), 2.0, #COLOR_MAROON)
            Next

        EndMode3D()

        DrawRectangle( 10, 10, 220, 70, Fade(#COLOR_SKYBLUE, 0.5))
        DrawRectangleLines( 10, 10, 220, 70, #COLOR_BLUE)

        DrawTextRaylib("First person camera default controls:", 20, 20, 10, #COLOR_BLACK)
        DrawTextRaylib("- Move with keys: W, A, S, D", 40, 40, 10, #COLOR_DARKGRAY)
        DrawTextRaylib("- Mouse move to look around", 40, 60, 10, #COLOR_DARKGRAY)

        DrawFPS(240,10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    ; If we want to have a screenshot
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/core_3d_camera_first_person.png")
    EndIf

Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
