;*******************************************************************************************
;*
;*   raylib [core] example - Initialize 3d camera mode
;*
;*   This example has been created using raylib 1.0 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2014 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit

;
; We import the module here, so we don't
; need to write the module name 'ray::'
; everywhere.
;
UseModule ray

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [core] example - 3d camera mode" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

; Define the camera to look into our 3d world
Define camera.Camera3D
InitVector3( @camera\position, 0.0, 10.0, 10.0 )    ; Camera position
InitVector3( @camera\target  , 0.0,  0.0,  0.0 )    ; Camera looking at point
InitVector3( @camera\up      , 0.0,  1.0,  0.0 )    ; Camera up vector (rotation towards target)
camera\fovy = 45.0                                  ; Camera field-of-view Y
camera\type = #CAMERATYPE_PERSPECTIVE               ; Camera mode type

Define cubePosition.Vector3                         ; PureBasic structures and variables are
                                                    ; initialized to 0

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    ; TODO: Update your variables here
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)
        
        BeginMode3D( @camera )

            DrawCube     ( @cubePosition, 2.0, 2.0, 2.0, #COLOR_RED)
            DrawCubeWires( @cubePosition, 2.0, 2.0, 2.0, #COLOR_MAROON)

            DrawGrid(10, 1.0)

        EndMode3D()

        DrawTextRaylib("Welcome to the third dimension!", 10, 40, 20, #COLOR_DARKGRAY)

        DrawFPS(10,10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    ; If we want to have a screenshot
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/core_3d_camera_mode.png")
    EndIf

Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
