;*******************************************************************************************
;*
;*   raylib [core] example - Windows drop files
;*
;*   This example only works on platforms that support drag & drop (Windows, Linux, OSX, Html5?)
;*
;*   This example has been created using raylib 1.3 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2015 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

; Enable config flags for resizable window and vertical synchro
SetConfigFlags(#FLAG_WINDOW_RESIZABLE | #FLAG_VSYNC_HINT)

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [core] example - drop files" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

Dim fileNames.s(0)
Define count, i

;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    If IsFileDropped()
        GetDroppedFiles( fileNames() )
        count = ArraySize(fileNames())
    EndIf
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        If count = 0
            DrawTextRaylib("Drop your files to this window!", 100, 40, 20, #COLOR_DARKGRAY)
        Else
            DrawTextRaylib("Dropped files:", 100, 40, 20, #COLOR_DARKGRAY)
            
            For i = 0 To count-1
                If i%2 = 0
                    DrawRectangle(0, 85 + 40*i, GetScreenWidth(), 40, Fade(#COLOR_LIGHTGRAY, 0.5))
                Else
                    DrawRectangle(0, 85 + 40*i, GetScreenWidth(), 40, Fade(#COLOR_LIGHTGRAY, 0.3))
                EndIf

                DrawTextRaylib(fileNames(i), 120, 100 + 40*i, 10, #COLOR_GRAY)
            Next

            DrawTextRaylib("Drop new files...", 100, 110 + 40*count, 20, #COLOR_DARKGRAY)
        EndIf

        DrawFPS(10,10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    ; If we want to have a screenshot
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/core_drop_files.png")
    EndIf

Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
ClearDroppedFiles()     ; Clear internal buffers

CloseWindowRaylib()     ; Close window And OpenGL context
;>-------------------------------------------------------------------------------------
