;*******************************************************************************************
;*
;*   raylib [core] example - Generate random values
;*
;*   This example has been created using raylib 1.1 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2014 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450


InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [core] example - generate random values" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

Define framesCounter                    ; Variable used to count frames
Define randValue = GetRandomValue(-8, 5); Get a random integer number between -8 and 5 (both included)

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    framesCounter + 1

    ; Every two seconds (120 frames) a new random value is generated
    If ((framesCounter/120) % 2) = 1
        randValue = GetRandomValue(-8, 5)
        framesCounter = 0
    EndIf
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        DrawTextRaylib("Every 2 seconds a new random value is generated:", 130, 100, 20, #COLOR_MAROON)

        DrawTextRaylib(Str(randValue), 360, 180, 80, #COLOR_LIGHTGRAY)

        DrawFPS(10,10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If ray::IsKeyPressed(ray::#KEY_F1)
        ray::TakeScreenshot("screenshots/core_random_values.png")
    EndIf
    
Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
