;*******************************************************************************************
;*
;*   raylib [core] example - Storage save/load values
;*
;*   This example has been created using raylib 1.4 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2015 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module


; NOTE: Storage positions must start with 0, directly related to file memory layout
Enumeration StorageData
    #STORAGE_POSITION_SCORE   = 0
    #STORAGE_POSITION_HISCORE = 1
EndEnumeration


; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [core] example - storage save/load values" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

Define framesCounter                    ; Variable used to count frames
Define score, hiscore

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    If IsKeyPressed(#KEY_R)
        score = GetRandomValue(1000, 2000)
        hiscore = GetRandomValue(2000, 4000)
    EndIf

    If IsKeyPressed(#KEY_ENTER)
        SaveStorageValue(#STORAGE_POSITION_SCORE, score)
        SaveStorageValue(#STORAGE_POSITION_HISCORE, hiscore)
    ElseIf IsKeyPressed(#KEY_SPACE)
        ; NOTE: If requested position could not be found, value 0 is returned
        score = LoadStorageValue(#STORAGE_POSITION_SCORE)
        hiscore = LoadStorageValue(#STORAGE_POSITION_HISCORE)
    EndIf

    framesCounter + 1
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        DrawTextRaylib("SCORE: "+score, 280, 130, 40, #COLOR_MAROON)
        DrawTextRaylib("HI-SCORE: "+hiscore, 210, 200, 50, #COLOR_BLACK)
        DrawTextRaylib("frames: "+framesCounter, 10, 10, 20, #COLOR_LIME)
        DrawTextRaylib("Press R to generate random numbers", 220, 40, 20, #COLOR_LIGHTGRAY)
        DrawTextRaylib("Press ENTER to SAVE values", 250, 310, 20, #COLOR_LIGHTGRAY)
        DrawTextRaylib("Press SPACE to LOAD values", 252, 350, 20, #COLOR_LIGHTGRAY)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If ray::IsKeyPressed(ray::#KEY_F1)
        ray::TakeScreenshot("screenshots/core_storage_values.png")
    EndIf
    
Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
