;*******************************************************************************************
;*
;*   raylib [core] example - World to screen
;*
;*   This example has been created using raylib 1.3 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2015 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [core] example - World to Screen" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

; Define the camera to look into our 3d world
Define camera.Camera3D
InitVector3( @camera\position, 10.0, 10.0, 10.0 )   ; Camera position
InitVector3( @camera\target  ,  0.0,  0.0,  0.0 )   ; Camera looking at point
InitVector3( @camera\up      ,  0.0,  1.0,  0.0 )   ; Camera up vector (rotation towards target)
camera\fovy = 45.0                                  ; Camera field-of-view Y
camera\type = #CAMERATYPE_PERSPECTIVE               ; Camera mode type

Define cubePosition.Vector3, vec.Vector3
Define cubeScreenPosition.Vector2

InitVector3(@vec, cubePosition\x, cubePosition\y + 2.5, cubePosition\z)

SetCameraMode(@camera, #CAMERAMODE_FREE)            ; Set a free camera mode

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    UpdateCamera( @camera )             ; Update camera

    ; Calculate cube screen space position (with a little offset to be in top)
    GetWorldToScreen(@cubeScreenPosition, @vec, @camera)
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)
        
        BeginMode3D(@camera)

            DrawCube(@cubePosition, 2.0, 2.0, 2.0, #COLOR_RED)
            DrawCubeWires(@cubePosition, 2.01, 2.01, 2.01, #COLOR_MAROON)

            DrawGrid(10, 1.0)

        EndMode3D()

        DrawTextRaylib("Enemy: 100 / 100", cubeScreenPosition\x - MeasureText("Enemy: 100 / 100", 20)/2,
                                           cubeScreenPosition\y, 20, #COLOR_BLACK)

        DrawTextRaylib("Text is always on top of the cube",
                       (#SCREEN_WIDTH - MeasureText("Text is always on top of the cube", 20))/2,
                       25, 20, #COLOR_GRAY)

        DrawFPS(10,10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    ; If we want to have a screenshot
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/core_world_screen.png")
    EndIf

Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
