;*******************************************************************************************
;*
;*   raylib [shapes] example - Colors palette
;*
;*   This example has been created using raylib 2.5 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2014-2019 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

#MAX_COLORS_COUNT = 21                  ; Number of colors available

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [shapes] example - colors palette" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

Define i.i

Dim colors(#MAX_COLORS_COUNT)
Dim colorNames.s(#MAX_COLORS_COUNT)
Dim colorsRecs.Rectangle(#MAX_COLORS_COUNT) ; Rectangles Array
Dim colorState(#MAX_COLORS_COUNT)           ; Color state: 0-DEFAULT, 1-MOUSE_HOVER

Restore ColorValues : For i = 0 To #MAX_COLORS_COUNT - 1 : Read.i colors(i)     : Next
Restore ColorNames  : For i = 0 To #MAX_COLORS_COUNT - 1 : Read.s colorNames(i) : Next

; Fills colorsRecs Data (For every rectangle)
For i = 0 To #MAX_COLORS_COUNT - 1
    colorsRecs(i)\x = 20 + 100*Mod(i,7) + 10*Mod(i,7)
    colorsRecs(i)\y = 80 + 100*Int(i/7) + 10*Int(i/7)
    colorsRecs(i)\width = 100
    colorsRecs(i)\height = 100
Next

Define.Vector2 mousePoint
Define fadeValue.f

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    GetMousePosition(@mousePoint)

    For i = 0 To #MAX_COLORS_COUNT - 1
        If CheckCollisionPointRec(@mousePoint, @colorsRecs(i))
            colorState(i) = 1
        Else
            colorState(i) = 0
        EndIf
    Next
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)
        
        DrawTextRaylib("raylib colors palette", 28, 42, 20, #COLOR_BLACK)
        DrawTextRaylib("press SPACE to see all colors", GetScreenWidth() - 180, GetScreenHeight() - 40, 10, #COLOR_GRAY)

        For i = 0 To #MAX_COLORS_COUNT - 1 ; Draw all rectangles
            If colorState(i) : fadeValue = 0.6 : Else : fadeValue = 1.0 : EndIf
            DrawRectangleRec(@colorsRecs(i), Fade(colors(i), fadeValue))

            If IsKeyDown(#KEY_SPACE) Or colorState(i)
                DrawRectangle(colorsRecs(i)\x, colorsRecs(i)\y + colorsRecs(i)\height - 26, colorsRecs(i)\width, 20, #COLOR_BLACK)
                DrawRectangleLinesEx(@colorsRecs(i), 6, Fade(#COLOR_BLACK, 0.3))
                DrawTextRaylib(colorNames(i), colorsRecs(i)\x + colorsRecs(i)\width - MeasureText(colorNames(i), 10) - 12,
                               colorsRecs(i)\y + colorsRecs(i)\height - 20, 10, colors(i))
            EndIf
        Next
        
        
        DrawFPS(10, 10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/shapes_colors_palette.png")
    EndIf
    
Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------

DataSection
    ColorValues:
    Data.i #COLOR_DARKGRAY, #COLOR_MAROON, #COLOR_ORANGE, #COLOR_DARKGREEN
    Data.i #COLOR_DARKBLUE, #COLOR_DARKPURPLE, #COLOR_DARKBROWN,#COLOR_GRAY
    Data.i #COLOR_RED, #COLOR_GOLD, #COLOR_LIME, #COLOR_BLUE
    Data.i #COLOR_VIOLET, #COLOR_BROWN, #COLOR_LIGHTGRAY, #COLOR_PINK
    Data.i #COLOR_YELLOW, #COLOR_GREEN, #COLOR_SKYBLUE, #COLOR_PURPLE, #COLOR_BEIGE
    ColorNames:
    Data.s "DARKGRAY", "MAROON", "ORANGE", "DARKGREEN", "DARKBLUE", "DARKPURPLE"
    Data.s "DARKBROWN", "GRAY", "RED", "GOLD", "LIME", "BLUE", "VIOLET", "BROWN"
    Data.s "LIGHTGRAY", "PINK", "YELLOW", "GREEN", "SKYBLUE", "PURPLE", "BEIGE"
EndDataSection

