;*******************************************************************************************
;*
;*   raylib [shapes] example - following eyes
;*
;*   This example has been created using raylib 2.5 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2013-2019 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [shapes] example - following eyes" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

Define.Vector2 scleraLeftPosition, scleraRightPosition
Define.Vector2 irisLeftPosition, irisRightPosition

InitVector2(@scleraLeftPosition, GetScreenWidth()/2 - 100, GetScreenHeight()/2)
InitVector2(@scleraRightPosition, GetScreenWidth()/2 + 100, GetScreenHeight()/2)
InitVector2(@irisLeftPosition, GetScreenWidth()/2 - 100, GetScreenHeight()/2)
InitVector2(@irisRightPosition, GetScreenWidth()/2 + 100, GetScreenHeight()/2)

Define.f scleraRadius = 80, irisRadius = 24, angle=0.0, dx = 0.0, dy = 0.0, dxx = 0.0, dyy = 0.0

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    GetMousePosition(@irisLeftPosition)
    GetMousePosition(@irisRightPosition)

    ; Check not inside the left eye sclera
    If Not CheckCollisionPointCircle(@irisLeftPosition, @scleraLeftPosition, scleraRadius - 20)
        dx = irisLeftPosition\x - scleraLeftPosition\x
        dy = irisLeftPosition\y - scleraLeftPosition\y

        angle = ATan2(dx, dy)

        dxx = (scleraRadius - irisRadius)*Cos(angle)
        dyy = (scleraRadius - irisRadius)*Sin(angle)

        irisLeftPosition\x = scleraLeftPosition\x + dxx
        irisLeftPosition\y = scleraLeftPosition\y + dyy
    EndIf

    ; Check not inside the right eye sclera
    If Not CheckCollisionPointCircle(@irisRightPosition, @scleraRightPosition, scleraRadius - 20)
        dx = irisRightPosition\x - scleraRightPosition\x
        dy = irisRightPosition\y - scleraRightPosition\y

        angle = ATan2(dx, dy)

        dxx = (scleraRadius - irisRadius)*Cos(angle)
        dyy = (scleraRadius - irisRadius)*Sin(angle)

        irisRightPosition\x = scleraRightPosition\x + dxx
        irisRightPosition\y = scleraRightPosition\y + dyy
    EndIf
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        DrawCircleV(@scleraLeftPosition, scleraRadius, #COLOR_LIGHTGRAY)
        DrawCircleV(@irisLeftPosition, irisRadius, #COLOR_BROWN)
        DrawCircleV(@irisLeftPosition, 10, #COLOR_BLACK)

        DrawCircleV(@scleraRightPosition, scleraRadius, #COLOR_LIGHTGRAY)
        DrawCircleV(@irisRightPosition, irisRadius, #COLOR_DARKGREEN)
        DrawCircleV(@irisRightPosition, 10, #COLOR_BLACK)

        DrawFPS(10, 10)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/shapes_following_eyes.png")
    EndIf
    
Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
