;*******************************************************************************************
;*
;*   raylib [text] example - raylib font loading and usage
;*
;*   NOTE: raylib is distributed with some free to use fonts (even for commercial pourposes!)
;*         To view details and credits for those fonts, check raylib license file
;*
;*   This example has been created using raylib 1.7 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2017 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

#MAX_FONTS = 8

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [text] example - raylib fonts" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

; NOTE: Textures MUST be loaded after Window initialization (OpenGL context is required)
Dim fonts.Font(#MAX_FONTS)

LoadFontRaylib(@fonts(0), "resources/fonts/alagard.png")
LoadFontRaylib(@fonts(1), "resources/fonts/pixelplay.png")
LoadFontRaylib(@fonts(2), "resources/fonts/mecha.png")
LoadFontRaylib(@fonts(3), "resources/fonts/setback.png")
LoadFontRaylib(@fonts(4), "resources/fonts/romulus.png")
LoadFontRaylib(@fonts(5), "resources/fonts/pixantiqua.png")
LoadFontRaylib(@fonts(6), "resources/fonts/alpha_beta.png")
LoadFontRaylib(@fonts(7), "resources/fonts/jupiter_crash.png")

Dim messages.s(#MAX_FONTS)
messages(0) = "ALAGARD FONT designed by Hewett Tsoi"
messages(1) = "PIXELPLAY FONT designed by Aleksander Shevchuk"
messages(2) = "MECHA FONT designed by Captain Falcon"
messages(3) = "SETBACK FONT designed by Brian Kent (AEnigma)"
messages(4) = "ROMULUS FONT designed by Hewett Tsoi"
messages(5) = "PIXANTIQUA FONT designed by Gerhard Grossmann"
messages(6) = "ALPHA_BETA FONT designed by Brian Kent (AEnigma)"
messages(7) = "JUPITER_CRASH FONT designed by Brian Kent (AEnigma)"

Dim spacings(#MAX_FONTS)
spacings(0) = 2
spacings(1) = 4
spacings(2) = 8
spacings(3) = 4
spacings(4) = 3
spacings(5) = 4
spacings(6) = 4
spacings(7) = 1

Define i, measuredText.Vector2

Dim positions.Vector2(#MAX_FONTS)
For i = 0 To #MAX_FONTS - 1
    MeasureTextEx(@measuredText, @fonts(i), messages(i), fonts(i)\baseSize*2, spacings(i))
    positions(i)\x = #SCREEN_WIDTH/2 - measuredText\x/2
    positions(i)\y = 60 + fonts(i)\baseSize + 45*i
Next

; Small Y position corrections
positions(3)\y + 8
positions(4)\y + 2
positions(7)\y - 8

Dim colors(#MAX_FONTS)
colors(0) = #COLOR_MAROON
colors(1) = #COLOR_ORANGE
colors(2) = #COLOR_DARKGREEN
colors(3) = #COLOR_DARKBLUE
colors(4) = #COLOR_DARKPURPLE
colors(5) = #COLOR_LIME
colors(6) = #COLOR_GOLD
colors(7) = #COLOR_RED

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
While Not WindowShouldClose()           ; Detect window close button or ESC key
    ; Update
    ;>---------------------------------------------------------------------------------
    ; TODO: Update your variables here
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        DrawTextRaylib("free fonts included with raylib", 250, 20, 20, #COLOR_DARKGRAY)
        DrawLine(220, 50, 590, 50, #COLOR_DARKGRAY)

        For i = 0 To #MAX_FONTS - 1
            DrawTextEx(@fonts(i), messages(i), @positions(i), fonts(i)\baseSize*2, spacings(i), colors(i))
        Next

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/text_raylib_fonts.png")
    EndIf
    
Wend

; De-Initialization
;>-------------------------------------------------------------------------------------
For i = 0 To #MAX_FONTS - 1
    UnloadFont(fonts(i))
Next

CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
