;*******************************************************************************************
;*
;*   raylib [textures] example - Background scrolling
;*
;*   This example has been created using raylib 2.0 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2019 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [textures] example - background scrolling" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

; NOTE: Textures MUST be loaded after Window initialization (OpenGL context is required)

; NOTE: Be careful, background width must be equal or bigger than screen width
; if not, texture should be draw more than two times for scrolling effect
Define.Texture2D background, midground, foreground

LoadTextureRaylib(@background, "resources/cyberpunk_street_background.png")
LoadTextureRaylib(@midground,  "resources/cyberpunk_street_midground.png")
LoadTextureRaylib(@foreground, "resources/cyberpunk_street_foreground.png")

Define.f scrollingBack = 0.0, scrollingMid = 0.0, scrollingFore = 0.0

Define.Vector2 v1, v2

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
Repeat
    ; Update
    ;>---------------------------------------------------------------------------------
    scrollingBack - 0.1
    scrollingMid  - 0.5
    scrollingFore - 1.0

    ; NOTE: Texture is scaled twice its size, so it sould be considered on scrolling
    If scrollingBack <= -background\width * 2 : scrollingBack = 0 : EndIf
    If scrollingMid  <= -midground\width  * 2 : scrollingMid  = 0 : EndIf
    If scrollingFore <= -foreground\width * 2 : scrollingFore = 0 : EndIf
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground( RGBA($05,$2C,$46,$FF) )

        ; Draw background image twice
        ; NOTE: Texture is scaled twice its size
        InitVector2(@v1, scrollingBack, 20)
        InitVector2(@v2, background\width*2 + scrollingBack, 20)
        DrawTextureEx(@background, @v1, 0.0, 2.0, #COLOR_WHITE)
        DrawTextureEx(@background, @v2, 0.0, 2.0, #COLOR_WHITE)

        ; Draw midground image twice
        InitVector2(@v1, scrollingMid, 20)
        InitVector2(@v2, midground\width*2 + scrollingMid, 20)
        DrawTextureEx(@midground, @v1, 0.0, 2.0, #COLOR_WHITE)
        DrawTextureEx(@midground, @v2, 0.0, 2.0, #COLOR_WHITE)

        ; Draw foreground image twice
        InitVector2(@v1, scrollingFore, 70)
        InitVector2(@v2, foreground\width*2 + scrollingFore, 70)
        DrawTextureEx(@foreground, @v1, 0.0, 2.0, #COLOR_WHITE)
        DrawTextureEx(@foreground, @v2, 0.0, 2.0, #COLOR_WHITE)

        DrawTextRaylib("BACKGROUND SCROLLING & PARALLAX", 10, 10, 20, #COLOR_RED)
        DrawTextRaylib("(c) Cyberpunk Street Environment by Luis Zuno (@ansimuz)",
                       #SCREEN_WIDTH  - 330,
                       #SCREEN_HEIGHT -  20,
                       10, #COLOR_RAYWHITE)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/textures_background_scrolling.png")
    EndIf
    
Until WindowShouldClose()               ; Detect window close button or ESC key

; De-Initialization
;>-------------------------------------------------------------------------------------
UnloadTexture(@background)              ; Unload background texture
UnloadTexture(@midground)               ; Unload midground texture
UnloadTexture(@foreground)              ; Unload foreground texture

CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
