;*******************************************************************************************
;*
;*   raylib [textures] example - Texture loading and drawing a part defined by a rectangle
;*
;*   This example has been created using raylib 1.3 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2014 Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

#MAX_FRAME_SPEED = 15
#MIN_FRAME_SPEED =  1

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [texture] example - texture rectangle" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

; NOTE: Textures MUST be loaded after Window initialization (OpenGL context is required)
Define scarfy.Texture2D
LoadTextureRaylib(@scarfy, "resources/scarfy.png")  ; Texture loading

Define position.Vector2
InitVector2(@position, 350.0, 280.0)
Define frameRec.Rectangle
InitRectangle(@frameRec, 0.0, 0.0, scarfy\width / 6.0, scarfy\height)

Define currentFrame, framesCounter, i
Define framesSpeed = 8                  ; Number of spritesheet frames shown by second

SetTargetFPS(60)                        ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
Repeat
    ; Update
    ;>---------------------------------------------------------------------------------
    framesCounter + 1

    If framesCounter >= 60/framesSpeed
        framesCounter = 0
        currentFrame + 1

        If currentFrame > 5
            currentFrame = 0
        EndIf

        frameRec\x = currentFrame * scarfy\width / 6.0
    EndIf

    If IsKeyPressed(#KEY_RIGHT)
        framesSpeed + 1
    ElseIf IsKeyPressed(#KEY_LEFT)
        framesSpeed - 1
    EndIf

    If framesSpeed > #MAX_FRAME_SPEED
        framesSpeed = #MAX_FRAME_SPEED
    ElseIf framesSpeed < #MIN_FRAME_SPEED
        framesSpeed = #MIN_FRAME_SPEED
    EndIf
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        DrawTexture(@scarfy, 15, 40, #COLOR_WHITE)
        DrawRectangleLines(15, 40, scarfy\width, scarfy\height, #COLOR_LIME)
        DrawRectangleLines(15 + frameRec\x, 40 + frameRec\y, frameRec\width, frameRec\height, #COLOR_RED)

        DrawTextRaylib("FRAME SPEED: ", 165, 210, 10, #COLOR_DARKGRAY)
        DrawTextRaylib(Str(framesSpeed)+"FPS", 575, 210, 10, #COLOR_DARKGRAY)
        DrawTextRaylib("PRESS RIGHT/LEFT KEYS to CHANGE SPEED!", 290, 240, 10, #COLOR_DARKGRAY)

        for i = 0 to #MAX_FRAME_SPEED - 1
            if i < framesSpeed : DrawRectangle(250 + 21*i, 205, 20, 20, #COLOR_RED) : endif
            DrawRectangleLines(250 + 21*i, 205, 20, 20, #COLOR_MAROON)
        next

        DrawTextureRec(@scarfy, @frameRec, @position, #COLOR_WHITE) ; Draw part of the texture

        DrawTextRaylib("(c) Scarfy sprite by Eiden Marsal", #SCREEN_WIDTH - 200, #SCREEN_HEIGHT - 20, 10, #COLOR_GRAY)

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/textures_rectangle.png")
    EndIf
    
Until WindowShouldClose()               ; Detect window close button or ESC key

; De-Initialization
;>-------------------------------------------------------------------------------------
UnloadTexture(@scarfy)                  ; Texture unloading

CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
