;*******************************************************************************************
;*
;*   raylib [textures] example - sprite explosion
;*
;*   This example has been created using raylib 2.5 (www.raylib.com)
;*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
;*
;*   Copyright (c) 2019 Anata and Ramon Santamaria (@raysan5)
;*
;*   Translated to PureBasic by Danilo Krahn, May 2020
;*
;********************************************************************************************
IncludePath "./../../"                  ; First we need to include
XIncludeFile "raylib.pbi"               ; the raylib-purebasic import

EnableExplicit                          ; All variables have to be defined before use

UseModule ray                           ; Import the module

#NUM_FRAMES = 8
#NUM_LINES  = 6

; Initialization
;>-------------------------------------------------------------------------------------
#SCREEN_WIDTH = 800
#SCREEN_HEIGHT = 450

InitWindow( #SCREEN_WIDTH, #SCREEN_HEIGHT, "raylib [textures] example - sprite explosion" )
If Not IsWindowReady()                  ; After creating the window, we check
    End                                 ; for errors at initialization.
EndIf                                   ; In case of error at init we end the program.

InitAudioDevice()                       ; Initialize audio device

Define fxBoom.Sound
LoadSoundRaylib(@fxBoom, "resources/boom.wav")      ; Load explosion sound

Define explosion.Texture2D
LoadTextureRaylib(@explosion, "resources/explosion.png")    ; Load explosion texture

; Init variables for animation
define frameWidth   = explosion\width  / #NUM_FRAMES    ; Sprite one frame rectangle width
define frameHeight  = explosion\height / #NUM_LINES     ; Sprite one frame rectangle height
define currentFrame = 0
define currentLine  = 0

Define frameRec.Rectangle
InitRectangle(@frameRec, 0, 0, frameWidth, frameHeight)

Define position.Vector2
Define active = #false
Define framesCounter

SetTargetFPS(120)                       ; Set our game to run at 60 frames-per-second
;>-------------------------------------------------------------------------------------

; Main game loop
Repeat
    ; Update
    ;>---------------------------------------------------------------------------------
    ; Check for mouse button pressed and activate explosion (if not active)
    if IsMouseButtonPressed(#MOUSE_LEFT_BUTTON) And not active
        GetMousePosition(@position)
        active = #true

        position\x - frameWidth  / 2
        position\y - frameHeight / 2

        PlaySoundRaylib(fxBoom)
    endif

    ; Compute explosion animation frames
    if active
        framesCounter + 1

        if framesCounter > 2
            currentFrame + 1

            if currentFrame >= #NUM_FRAMES
                currentFrame = 0
                currentLine  + 1

                if currentLine >= #NUM_LINES
                    currentLine = 0
                    active = #false
                endif
            endif

            framesCounter = 0
        endif
    endif

    frameRec\x = frameWidth*currentFrame
    frameRec\y = frameHeight*currentLine
    ;>---------------------------------------------------------------------------------

    ; Draw
    ;>---------------------------------------------------------------------------------
    BeginDrawing()
    
        ClearBackground(#COLOR_RAYWHITE)

        ; Draw explosion required frame rectangle
        if active
            DrawTextureRec(@explosion, @frameRec, @position, #COLOR_WHITE)
        endif

    EndDrawing()
    ;>---------------------------------------------------------------------------------
    
    If IsKeyPressed(#KEY_F1)
        TakeScreenshot("screenshots/textures_sprite_explosion.png")
    EndIf
    
Until WindowShouldClose()               ; Detect window close button or ESC key

; De-Initialization
;>-------------------------------------------------------------------------------------
UnloadTexture(@explosion)               ; Unload texture
UnloadSound(@fxBoom)                    ; Unload sound

CloseAudioDevice()                      ; Close audio device

CloseWindowRaylib()                     ; Close window and OpenGL context
;>-------------------------------------------------------------------------------------
